package at.jku.risc.stout.urauc;

/*
import com.google.gwt.event.dom.client.*;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.dom.client.*;
*/

import java.io.PrintStream;
import java.util.Set;

import at.jku.risc.stout.urauc.algo.AlignFnc;
import at.jku.risc.stout.urauc.algo.DebugLevel;
import at.jku.risc.stout.urauc.data.InputParser;
import at.jku.risc.stout.urauc.data.atom.TermAtom;
import at.jku.risc.stout.urauc.util.ControlledException;

/**
 * HelloWorld application.
 */
public class AntiUnifyGWT implements EntryPoint {

	private static final int MAX_LINES = 5000;
	// Jitter for MAX_LINES = 4900 - 5100
	private static final int MAX_LINES_VARIANZ = 100;
	private static final String PRINT_HEAD = "";
	private static final String PRINT_TAIL = "DONE";

	private final PrintStream out = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();
		long count = 0L;

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n') {
				count++;
				flush();
			}
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n') {
				count++;
				flush();
			}
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			String lineNum = "000000" + count + ": ";
			if (buffer.length() > 0)
				display(lineNum.substring(lineNum.length() - 8,
						lineNum.length())
						+ buffer.toString());
			buffer.setLength(0);
		}
		public void close() {
			flush();
			count = 0L;
		}
	};
	private final PrintStream info = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n')
				flush();
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n')
				flush();
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			if (buffer.length() > 0)
				display("INFO:  " + buffer.toString(), "gray");
			buffer.setLength(0);
		}
		public void close() {
			flush();
		}
	};
	private final PrintStream err = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n')
				flush();
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n')
				flush();
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			if (buffer.length() > 0)
				display("ERROR: " + buffer.toString(), "red");
			buffer.setLength(0);
		}
		public void close() {
			flush();
		}
	};

	public void btnClicked() {
		reset();
		if (PRINT_HEAD.length() > 0)
			display(PRINT_HEAD, "gray");
		try {
			String aupsIn = getParameter("problemset");
			DebugLevel debugLevel = DebugLevel.VERBOSE;
			try {
				debugLevel = DebugLevel.valueOf(getParameter("debugmode"));
			} catch (Exception ignored) {
			}
			boolean iterateAll = "true".equals(getParameter("iterateall"));
			boolean justify = "true".equals(getParameter("justify"));
			boolean applyResH = "true".equals(getParameter("applyresh"));
			final boolean equalityFilter = "true".equals(getParameter("equalityfilter"));
			boolean commRoot = "true".equals(getParameter("commutativeroot"));
			Set<TermAtom> commFnc = InputParser
					.parseAtomSet(getParameter("commutativesymbols"));
			Set<TermAtom> assocFnc = InputParser
					.parseAtomSet(getParameter("associativesymbols"));
			AlignFnc aFnc = null;
			String aFncString = getParameter("alignfnc");
			if (aFncString == null || aFncString.length() == 0)
				aFncString = "at.jku.risc.stout.urauc.algo.AlignFncLAA";
			if ("at.jku.risc.stout.urauc.algo.AlignFncLAA".equals(aFncString)) {
				aFnc = new at.jku.risc.stout.urauc.algo.AlignFncLAA();
			} else if ("at.jku.risc.stout.urauc.algo.AlignFncInput".equals(aFncString)) {
				aFnc = new at.jku.risc.stout.urauc.algo.AlignFncInput();
			} else {
				throw new ControlledException("Alignment computation function invalid: " + aFncString);
			}
			int minLen = 0;
			try {
				minLen = Integer.parseInt(getParameter("minlen"));
			} catch (Exception e) {
			}
			int upperBound = 0;
			try {
				upperBound = Integer.parseInt(getParameter("upperbound"));
			} catch (Exception e) {
			}

			info.println("Alignment computation function = " + aFncString);
			info.println("Justify computed generalization = " + justify);
			info.println("Iterate all possibilities = " + iterateAll);
			if (!"true".equals(getParameter("ignorealgebraicaxioms"))) {
				info.println("Minimum alignment length = " + minLen);
				info.println("Upper bound for commutative arrangement iteration = " + upperBound);
				info.println("Apply equality filter = " + equalityFilter);
			}
			info.println("Output format = " + debugLevel);
			info.println();
			if (!"true".equals(getParameter("ignorealgebraicaxioms"))) {
				info.println("Commutative root hedge = " + commRoot);
				info.println("Commutative symbols = " + commFnc);
				info.println("Associative symbols = " + assocFnc);
				info.println();
			}
			info.println("PROBLEM:");
			new AntiUnifyConsole(debugLevel, info, out).exec(
					aupsIn, getParameter("alignment"), iterateAll,
					justify, applyResH, equalityFilter, commRoot,
					commFnc, assocFnc, aFnc, minLen, upperBound);
		} catch (ControlledException ex) {
			err.println(ex.getMessage() + " (" + ex.getClass().getSimpleName()
					+ ")");
		} catch (Throwable ex) {
			ex.printStackTrace(err);
		}

		info.flush();
		out.flush();
		err.flush();
		display(PRINT_TAIL, "gray");
	}

	private void reset() {
	}

	public String getParameter(String name) {
		String ret = getTextVal(name);
		if (ret == null) ret = getSelectVal(name);
		if (ret == null) ret = getCheckedVal(name);
		return ret;
	}

	private String getTextVal(String name) {
		if ("problemset".equals(name))
			return "f(c), f(f(g(a, a)), a, a) =^= c, f(g(b, b, b), b, b, b)";
		if ("debugmode".equals(name))
			return DebugLevel.VERBOSE.toString();
		return null;
	}

	private String getCheckedVal(String name) {
		return null;
	}

	private String getSelectVal(String name) {
		return null;
	}

	private static void display(Object obj, String... args) {
		System.out.print(obj);
	}

/*
	private Document gwtDoc;
	private Element gwtContainer;
	private Button gwtButton;
	private NodeCollection<Element> gwtFormElems;

	public void onModuleLoad() {
		gwtContainer = RootPanel.get().getElement();
		gwtDoc = gwtContainer.getOwnerDocument();
		gwtContainer = gwtDoc.getElementById("gwtconsole");
		gwtButton = Button.wrap(gwtDoc.getElementById("gwtbutton"));
		gwtFormElems = gwtButton.getElement().<ButtonElement> cast().getForm().getElements();
		gwtButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				btnClicked();
			}
		});
	}

	private void display(Object obj, String color) {
                Element gwtContainerParent = gwtContainer;
		gwtContainer = gwtDoc.createSpanElement();
		gwtContainer.getStyle().setColor(color);
		gwtContainerParent.appendChild(gwtContainer);
		display(obj);
                gwtContainer = gwtContainerParent;
        }
	private void display(Object obj) {
		gwtContainer.appendChild(gwtDoc.createTextNode(String.valueOf(obj)));
	}


	public String getTextVal(String name) {
		Element el = gwtFormElems.getNamedItem(name);
		if (InputElement.is(el)) {
			InputElement in = InputElement.as(el);
			if ("checkbox".equalsIgnoreCase(in.getType()) || "radio".equalsIgnoreCase(in.getType()))
				return null;
			return in.getValue();
		}
		if (TextAreaElement.is(el))
			return TextAreaElement.as(el).getValue();
		return null;
	}

	public String getCheckedVal(String name) {
		for (int i = 0, n = gwtFormElems.getLength(); i < n; i++) {
			Element el = gwtFormElems.getItem(i);
			if (InputElement.is(el)) {
				InputElement in = InputElement.as(el);
				if (in.isChecked() && name.equals(in.getName()))
					return in.getValue();
			}
		}
		return null;
	}

	public String getSelectVal(String name) {
		Element el = gwtFormElems.getNamedItem(name);
		if (SelectElement.is(el)) {
			SelectElement sel = SelectElement.as(el);
			if (sel.getLength() == 0)
				return null;
			int selIdx = sel.getSelectedIndex();
			if (selIdx == -1 && !sel.isMultiple())
				selIdx = 0;
			OptionElement selOpt = sel.getOptions().getItem(selIdx);
			return selOpt.getValue();
		}
		return null;
	}

	public void reset() {
		info.close();
		out.close();
		err.close();
		gwtContainer.removeAllChildren();
	}
*/

	public static void main(String[] args) {
		new AntiUnifyGWT().btnClicked();
	}

	// Web worker
	public final native void postMessage(String message) /*-{
		self.postMessage(message);
	}-*/;
}

