/*
 * Copyright 2014 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package at.jku.risc.stout.urauc;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Set;

import at.jku.risc.stout.urauc.algo.AlignFnc;
import at.jku.risc.stout.urauc.algo.AlignFncLAA;
import at.jku.risc.stout.urauc.algo.AntiUnify;
import at.jku.risc.stout.urauc.algo.AntiUnifyProblem;
import at.jku.risc.stout.urauc.algo.AntiUnifySystem;
import at.jku.risc.stout.urauc.algo.DebugLevel;
import at.jku.risc.stout.urauc.algo.JustificationException;
import at.jku.risc.stout.urauc.data.EquationSystem;
import at.jku.risc.stout.urauc.data.Hedge;
import at.jku.risc.stout.urauc.data.InputParser;
import at.jku.risc.stout.urauc.data.atom.FunctionSymbol;
import at.jku.risc.stout.urauc.data.atom.TermAtom;
import at.jku.risc.stout.urauc.data.atom.Variable;
import at.jku.risc.stout.urauc.util.ControlledException;
import at.jku.risc.stout.urauc.util.DataStructureFactory;

public class TestCommutativeExample {

	public static void main(String[] args) throws IOException,
			JustificationException, ControlledException {
		Set<TermAtom> comm = DataStructureFactory.$.newSet();
		comm.add(new FunctionSymbol("f"));
//		comm.add(new FunctionSymbol("g"));
//		comm.add(new FunctionSymbol("h"));
//		comm.add(new FunctionSymbol("X"));
//		comm.add(new FunctionSymbol("F"));
		Set<TermAtom> assoc = DataStructureFactory.$.newSet();
		// assoc.add(new FunctionSymbol("f"));

//		Reader in1 = new StringReader("f(g(c,a),b), f(g(c,a),b), f(g(c,a),b)");
//		Reader in2 = new StringReader("f(b,f(a,a,a,c)), f(b,f(a,a,a,c))");
//		 Reader in1 = new StringReader(
//		 "f(a,b,c,f(a,b,c,d,e),f(a,b,c,d,e),f(a,b,c,d,e),d,e)");
//		 Reader in2 = new StringReader(
//		 "f(a,b,c,f(a,b,c,d,e),f(a,d,e),f(a,b,c),d)");
//		 Reader in1 = new StringReader("f(c), f(f(g(a, a)), a, a)");
//		 Reader in2 = new StringReader("f(g(b, b, b), b, b, b),c");
//		 Reader in1 = new StringReader("f(g(c,a),b)");
//		 Reader in2 = new StringReader("f(b,f(a,a,a,c))");
//		 Reader in1 = new StringReader("a,f(b,a,a,a)");
//		 Reader in2 = new StringReader("f(a,a,b),a");
//		 Reader in1 = new StringReader(
//		 "f(c,d,x,g(Y,x,b),h(y,X,X(f(),g),F(f,g))),a,l,j,k");
//		 Reader in2 = new StringReader("x,g(f(b,c,d,f(b,b,b)),g(a,b)),g,f");
//		 Reader in1 = new StringReader(
//		 "f(a,b,c),a,b");
//		 Reader in2 = new StringReader("g(c,a),a");
		 Reader in1 = new StringReader("0,f(0,2)");
		 Reader in2 = new StringReader("1,g(2,1)");
		// Reader in1 = new StringReader(
		// "a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a(a()))))))))))))))))))))))))))))");
		// Reader in2 = new
		// StringReader("a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a");
		// Reader in1 = new StringReader("a");
		// Reader in2 = new StringReader("b");
		boolean iterateAll = true;

		AlignFnc aFnc = new AlignFncLAA();
		EquationSystem<AntiUnifyProblem> eqSys = new EquationSystem<AntiUnifyProblem>() {
			public AntiUnifyProblem newEquation() {
				return new AntiUnifyProblem();
			}
		};
		new InputParser<AntiUnifyProblem>(eqSys).parseHedgeEquation(in1, in2);
		for (AntiUnifyProblem aup : eqSys) {
			aup.makeCommutativ(comm, true);
			aup.makeAssociative(assoc);
		}

		final Set<Hedge> filter = DataStructureFactory.$.newSet();
		new AntiUnify(aFnc, eqSys, 0, 10000000, true, DebugLevel.VERBOSE) {
			public void callback(AntiUnifySystem res, Variable var) {
				Hedge result = res.getSigma().get(var);
				result.normalizeVarNames(new HashMap<String, String>());
				filter.add(result);
			};
		}.antiUnify(iterateAll, true, System.out);
		System.out.println();
		System.out.println(filter.size() + " FILTERED RESULTS:");
		for (Hedge result : filter)
			System.out.println(result);
	}
}
