/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urauc.algo;

import java.io.PrintStream;
import java.util.Iterator;

import at.jku.risc.stout.urauc.data.Alignment;
import at.jku.risc.stout.urauc.data.TermAtomList;
import at.jku.risc.stout.urauc.data.atom.TermAtom;

/**
 * This is the common super class for all alignment computation functions, which
 * compute a list of alignments for two given sequences (
 * {@linkplain TermAtomList}s) of {@linkplain TermAtom}s.
 * 
 * @author Alexander Baumgartner
 */
public abstract class AlignFnc {
	/**
	 * Debugging flag
	 */
	public static DebugLevel debugLevel = DebugLevel.SILENT;

	/**
	 * Returns a fresh new instance of this alignment computation function.
	 */
	public abstract AlignFnc newInstance();

	/**
	 * Initialize the computation. This method is called once before the
	 * iterations begin. For each pair, the user has to iterate over the
	 * alignment computation function {@linkplain AlignFnc} and for each
	 * iteration the anti-unification algorithm should be called.
	 * 
	 * @throws MaximumIterationException
	 *             If the specified upper bound has been reached. Any value
	 *             smaller than 1 disables the upper bound.
	 */
	public abstract AlignmentIterator getIterator(AntiUnifyProblem aup,
			int minAlignLen, int upperBound, boolean iterateAll, PrintStream out)
			throws MaximumIterationException;

	public interface AlignmentIterator extends Iterator<Alignment> {
		public int getCommutativeArrangementSize();

		public int getMaxAlignmentLen();
	}
}