/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urauc.data;

/**
 * Interface for equations which consist of two {@linkplain TermNode}s.
 * 
 * @author Alexander Baumgartner
 */
public interface Equation {
	/**
	 * Adds a {@linkplain TermNode} to the left hedge of the equation.
	 */
	public void addLeft(TermNode node);

	/**
	 * Adds a {@linkplain TermNode} to the right hedge of the equation.
	 */
	public void addRight(TermNode node);

	/**
	 * The left hedge of the equation.
	 */
	public Hedge getLeft();

	/**
	 * The right hedge of the equation.
	 */
	public Hedge getRight();

	/**
	 * This method is called automatically by the input parser after a new
	 * equation has been created. Override this method to implement an
	 * initialization for every newly generated equation.
	 */
	public void init();
}