/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urauc.data.atom;

import at.jku.risc.stout.urauc.data.NodeFactory;

/**
 * A function variable can be substituted by a bounded context, that is an
 * application of a function to a hedge with a hole in it. We do not technically
 * distinguish between context variables and function variables. The
 * implementation of the anti-unification algorithm guarantees that those
 * restrictions are fulfilled. Furthermore, different prefixes and suffixes are
 * defined in {@linkplain NodeFactory}
 * 
 * @author Alexander Baumgartner
 */
public class FunctionVar extends ContextVar {
	public FunctionVar(String name) {
		super(name);
	}

	protected String getPrefixFresh() {
		return NodeFactory.PREFIX_FreshFncVar;
	}

	protected String getSuffixFresh() {
		return NodeFactory.SUFFIX_FreshFncVar;
	}
}
